------------------------------------------------------------------------------
--                             G N A T C O L L                              --
--                                                                          --
--                     Copyright (C) 2003-2020, AdaCore                     --
--                                                                          --
-- This library is free software;  you can redistribute it and/or modify it --
-- under terms of the  GNU General Public License  as published by the Free --
-- Software  Foundation;  either version 3,  or (at your  option) any later --
-- version. This library is distributed in the hope that it will be useful, --
-- but WITHOUT ANY WARRANTY;  without even the implied warranty of MERCHAN- --
-- TABILITY or FITNESS FOR A PARTICULAR PURPOSE.                            --
--                                                                          --
-- As a special exception under Section 7 of GPL version 3, you are granted --
-- additional permissions described in the GCC Runtime Library Exception,   --
-- version 3.1, as published by the Free Software Foundation.               --
--                                                                          --
-- You should have received a copy of the GNU General Public License and    --
-- a copy of the GCC Runtime Library Exception along with this program;     --
-- see the files COPYING3 and COPYING.RUNTIME respectively.  If not, see    --
-- <http://www.gnu.org/licenses/>.                                          --
--                                                                          --
------------------------------------------------------------------------------

--  Bindings to error handling functions

package GNATCOLL.Python.Errors is

   function PyErr_Occurred return PyObject;
   pragma Import (C, PyErr_Occurred, "PyErr_Occurred");
   --  Return value: Borrowed reference.
   --
   --  Test whether the error indicator is set. If set, return the exception
   --  type (the first argument to the last call to one of the PyErr_Set*()
   --  functions or to PyErr_Restore()). If not set, return NULL. You do not
   --  own a reference to the return value, so you do not need to Py_DECREF()
   --  it.
   --
   --  Note: Do not compare the return value to a specific exception. Use
   --  PyErr_ExceptionMatches() instead. The comparison could easily fail
   --  since the exception may be an instance instead of a class,
   --  in the case of a class exception, or it may be a subclass of the
   --  expected exception.

end GNATCOLL.Python.Errors;
